@@ -2,11 +2,12 @@
 
 __author__ = """François Durand"""
 __email__ = 'fradurand@gmail.com'
-__version__ = '0.5.1'
+__version__ = '0.14.0'
 
 
 # Utils
 from poisson_approval.utils.DictPrintingInOrder import DictPrintingInOrder
+from poisson_approval.utils.DictPrintingInOrderIgnoringNone import DictPrintingInOrderIgnoringNone
 from poisson_approval.utils.DictPrintingInOrderIgnoringZeros import DictPrintingInOrderIgnoringZeros
 from poisson_approval.utils.SetPrintingInOrder import SetPrintingInOrder
 from poisson_approval.utils.Util import *
@@ -37,6 +38,9 @@ from poisson_approval.events.EventTrio2t import EventTrio2t
 
 # Best response
 from poisson_approval.best_response.BestResponse import BestResponse
+from poisson_approval.best_response.BestResponseAntiPlurality import BestResponseAntiPlurality
+from poisson_approval.best_response.BestResponseApproval import BestResponseApproval
+from poisson_approval.best_response.BestResponsePlurality import BestResponsePlurality
 
 # Tau-vector
 from poisson_approval.tau_vector.TauVector import TauVector
@@ -49,6 +53,7 @@ from poisson_approval.strategies.StrategyOrdinal import StrategyOrdinal
 
 # Profiles
 from poisson_approval.profiles.Profile import Profile
+from poisson_approval.profiles.ProfileDiscrete import ProfileDiscrete
 from poisson_approval.profiles.ProfileOrdinal import ProfileOrdinal
 from poisson_approval.profiles.ProfileCardinal import ProfileCardinal
 from poisson_approval.profiles.ProfileTwelve import ProfileTwelve
@@ -63,6 +68,8 @@ from poisson_approval.generators.GeneratorExample import GeneratorExample
 from poisson_approval.generators.GeneratorProfileOrdinalGridUniform import GeneratorProfileOrdinalGridUniform
 from poisson_approval.generators.GeneratorProfileOrdinalUniform import GeneratorProfileOrdinalUniform
 from poisson_approval.generators.GeneratorProfileOrdinalVariations import GeneratorProfileOrdinalVariations
+from poisson_approval.generators.GeneratorProfileHistogramSinglePeakedUniform \
+    import GeneratorProfileHistogramSinglePeakedUniform
 from poisson_approval.generators.GeneratorProfileHistogramUniform import GeneratorProfileHistogramUniform
 from poisson_approval.generators.GeneratorStrategyOrdinalUniform import GeneratorStrategyOrdinalUniform
 from poisson_approval.generators.GeneratorStrategyThresholdUniform import GeneratorStrategyThresholdUniform
@@ -71,3 +78,4 @@ from poisson_approval.generators.GeneratorTauVectorUniform import GeneratorTauVe
 
 # Meta-analysis
 from poisson_approval.meta_analysis.NiceStatsProfileOrdinal import NiceStatsProfileOrdinal
+from poisson_approval.meta_analysis.ternary_plots import TernaryAxesSubplotPoisson, ternary_figure
