@@ -7,6 +7,11 @@ from poisson_approval.strategies.StrategyThreshold import StrategyThreshold
 class GeneratorStrategyThresholdUniform:
     """A generator of threshold strategies (:class:`StrategyThreshold`) following the uniform distribution.
 
+    Parameters
+    ----------
+    kwargs : keyword arguments
+        These additional arguments will be passed directly to :class:`StrategyThreshold`.
+
     Notes
     -----
     Each threshold is drawn uniformly in the interval [0, 1].
@@ -22,6 +27,9 @@ bac: utility-dependent (0.6027633760716439), bca: utility-dependent (0.544883182
 cab: utility-dependent (0.4236547993389047), cba: utility-dependent (0.6458941130666561)>
     """
 
+    def __init__(self, **kwargs):
+        self.kwargs = kwargs
+
     def __call__(self):
         """
         Returns
@@ -29,4 +37,4 @@ cab: utility-dependent (0.4236547993389047), cba: utility-dependent (0.645894113
         StrategyThreshold
             A strategy.
         """
-        return StrategyThreshold({ranking: np.random.rand() for ranking in RANKINGS})
+        return StrategyThreshold({ranking: np.random.rand() for ranking in RANKINGS}, **self.kwargs)
