@@ -29,6 +29,11 @@ class ProfileHistogram(ProfileCardinal):
         Whether a warning should be issued if the input distribution is not normalized.
     ratio_sincere : Number
         The ratio of sincere voters, in the interval [0, 1]. This is used for :meth:`tau`.
+    ratio_fanatic : Number
+        The ratio of fanatic voters, in the interval [0, 1]. This is used for :meth:`tau`. The sum of `ratio_sincere`
+        and `ratio_fanatic` must not exceed 1.
+    voting_rule : str
+        The voting rule. Possible values are ``APPROVAL``, ``PLURALITY`` and ``ANTI_PLURALITY``.
 
     Notes
     -----
@@ -72,56 +77,51 @@ class ProfileHistogram(ProfileCardinal):
         >>> strategy = StrategyThreshold({'abc': 0, 'bac': 1, 'cab': Fraction(1, 2)}, profile=profile)
         >>> print(profile.tau_sincere)
         <a: 1/20, ab: 1/20, ac: 1/10, b: 3/5, c: 1/5> ==> b
+        >>> print(profile.tau_fanatic)
+        <a: 1/10, b: 3/5, c: 3/10> ==> b
         >>> print(profile.tau_strategic(strategy))
         <ab: 1/10, ac: 1/10, b: 3/5, c: 1/5> ==> b
         >>> print(profile.tau(strategy))
         <ab: 1/10, ac: 1/10, b: 3/5, c: 1/5> ==> b
         >>> profile.is_equilibrium(strategy)
         EquilibriumStatus.EQUILIBRIUM
-        >>> cycle = profile.iterated_voting_strategies(StrategyThreshold({'abc': .5, 'bac': .5, 'cab': .5}),  100)['responses']
+        >>> strategy_ini = StrategyThreshold({'abc': .5, 'bac': .5, 'cab': .5})
+        >>> cycle = profile.iterated_voting(strategy_ini, 100)['cycle_strategies']
         >>> len(cycle)
         1
         >>> print(cycle[0])
-        <abc: ab, bac: utility-dependent (0.7199316142046179), cab: utility-dependent (0.2800683857953819)> ==> b
+        <abc: ab, bac: utility-dependent (0.7199316142046179), cab: utility-dependent (0.28006838579538196)> ==> b
+        >>> limit_strategy = profile.fictitious_play(strategy_ini, 100, perception_update_ratio=1)['strategy']
+        >>> print(limit_strategy)
+        <abc: ab, bac: utility-dependent (0.7199316142046179), cab: utility-dependent (0.28006838579538196)> ==> b
     """
 
-    def __init__(self, d_ranking_share, d_ranking_histogram, normalization_warning=True, ratio_sincere=0):
+    def __init__(self, d_ranking_share, d_ranking_histogram, normalization_warning=True,
+                 ratio_sincere=0, ratio_fanatic=0, voting_rule=APPROVAL):
         """
-            >>> profile = ProfileHistogram(d_ranking_share={'non_existing_ranking': 1},
-            ...                            d_ranking_histogram={'abc': [1]})
-            Traceback (most recent call last):
-            ValueError: Unknown key: non_existing_ranking
             >>> profile = ProfileHistogram(d_ranking_share={'abc': 1},
             ...                            d_ranking_histogram={'non_existing_ranking': [1]})
             Traceback (most recent call last):
-            ValueError: Unknown key: non_existing_ranking
+            KeyError: 'non_existing_ranking'
         """
-        super().__init__(ratio_sincere=ratio_sincere)
-        # Populate the dictionary and check for typos in the input
-        self._d_ranking_share = DictPrintingInOrderIgnoringZeros()
-        self.d_ranking_histogram = DictPrintingInOrderIgnoringZeros()
+        super().__init__(ratio_sincere=ratio_sincere, ratio_fanatic=ratio_fanatic, voting_rule=voting_rule)
+        # Populate the dictionary (and check for typos in the input)
+        self._d_ranking_share = DictPrintingInOrderIgnoringZeros({ranking: 0 for ranking in RANKINGS})
+        self.d_ranking_histogram = DictPrintingInOrderIgnoringZeros({ranking: np.array([]) for ranking in RANKINGS})
         for ranking, share in d_ranking_share.items():
-            if ranking in RANKINGS:
-                self._d_ranking_share[ranking] = share
-            else:
-                raise ValueError('Unknown key: ' + ranking)
+            self._d_ranking_share[ranking] += share
         for ranking, histogram in d_ranking_histogram.items():
             if ranking in RANKINGS:
                 self.d_ranking_histogram[ranking] = np.array(histogram)
             else:
-                raise ValueError('Unknown key: ' + ranking)
-        for ranking in RANKINGS:
-            if ranking not in self._d_ranking_share:
-                self._d_ranking_share[ranking] = 0
-            if ranking not in self.d_ranking_histogram:
-                self.d_ranking_histogram[ranking] = np.array([])
+                raise KeyError('%s' % ranking)
         # Normalize if necessary
         total = sum(self._d_ranking_share.values())
         if not isclose(total, 1.):
             if normalization_warning:
                 warnings.warn("Warning: profile is not normalized, I will normalize it.")
             for ranking in self._d_ranking_share.keys():
-                self._d_ranking_share[ranking] = self._d_ranking_share[ranking] / total
+                self._d_ranking_share[ranking] /= total
         for ranking, histogram in self.d_ranking_histogram.items():
             if len(histogram) == 0:
                 continue
@@ -208,9 +208,37 @@ class ProfileHistogram(ProfileCardinal):
             return share_ranking * (np.sum(histogram[0:k]) + histogram[k] * (u * n_bins - k))
 
     def __repr__(self):
-        return 'ProfileHistogram(%r, %r)' % (self.d_ranking_share, self.d_ranking_histogram)
+        """
+            >>> from fractions import Fraction
+            >>> profile = ProfileHistogram(
+            ...     {'abc': Fraction(1, 10), 'bac': Fraction(6, 10), 'cab': Fraction(3, 10)},
+            ...     {'abc': [1], 'bac': [1, 0], 'cab': [Fraction(2, 3), 0, 0, 0, 0, 0, 0, 0, 0, Fraction(1, 3)]},
+            ...     ratio_sincere=Fraction(1, 10), ratio_fanatic=Fraction(2, 10))
+            >>> profile
+            ProfileHistogram({'abc': Fraction(1, 10), 'bac': Fraction(3, 5), 'cab': Fraction(3, 10)}, \
+{'abc': array([1]), 'bac': array([1, 0]), 'cab': array([Fraction(2, 3), 0, 0, 0, 0, 0, 0, 0, 0, Fraction(1, 3)],
+                  dtype=object)}, ratio_sincere=Fraction(1, 10), ratio_fanatic=Fraction(1, 5))
+        """
+        arguments = '%r, %r' % (self.d_ranking_share, self.d_ranking_histogram)
+        if self.ratio_sincere > 0:
+            arguments += ', ratio_sincere=%r' % self.ratio_sincere
+        if self.ratio_fanatic > 0:
+            arguments += ', ratio_fanatic=%r' % self.ratio_fanatic
+        if self.voting_rule != APPROVAL:
+            arguments += ', voting_rule=%r' % self.voting_rule
+        return 'ProfileHistogram(%s)' % arguments
 
     def __str__(self):
+        """
+            >>> from fractions import Fraction
+            >>> profile = ProfileHistogram(
+            ...     {'abc': Fraction(1, 10), 'bac': Fraction(6, 10), 'cab': Fraction(3, 10)},
+            ...     {'abc': [1], 'bac': [1, 0], 'cab': [Fraction(2, 3), 0, 0, 0, 0, 0, 0, 0, 0, Fraction(1, 3)]},
+            ...     ratio_sincere=Fraction(1, 10), ratio_fanatic=Fraction(2, 10))
+            >>> print(profile)
+            <abc: 1/10 [1], bac: 3/5 [1 0], cab: 3/10 [Fraction(2, 3) 0 0 0 0 0 0 0 0 Fraction(1, 3)]> \
+(Condorcet winner: b) (ratio_sincere: 1/10) (ratio_fanatic: 1/5)
+        """
         result = '<' + ', '.join([
             '%s: %s %s' % (ranking, self.d_ranking_share[ranking], self.d_ranking_histogram[ranking])
             for ranking in sorted(self.d_ranking_share)
@@ -218,6 +246,12 @@ class ProfileHistogram(ProfileCardinal):
         ]) + '>'
         if self.is_profile_condorcet:
             result += ' (Condorcet winner: %s)' % self.condorcet_winners
+        if self.ratio_sincere > 0:
+            result += ' (ratio_sincere: %s)' % self.ratio_sincere
+        if self.ratio_fanatic > 0:
+            result += ' (ratio_fanatic: %s)' % self.ratio_fanatic
+        if self.voting_rule != APPROVAL:
+            result += ' (%s)' % self.voting_rule
         return result
 
     def _repr_pretty_(self, p, cycle):  # pragma: no cover
@@ -249,13 +283,16 @@ class ProfileHistogram(ProfileCardinal):
         """
         return (isinstance(other, ProfileHistogram)
                 and self.d_ranking_share == other.d_ranking_share
-                and self.d_ranking_histogram == self.d_ranking_histogram)
+                and self.d_ranking_histogram == self.d_ranking_histogram
+                and self.ratio_sincere == other.ratio_sincere
+                and self.ratio_fanatic == other.ratio_fanatic
+                and self.voting_rule == other.voting_rule)
 
     # Standardized version of the profile (makes it unique, up to permutations)
 
     @cached_property
     def standardized_version(self):
-        """ProfileTwelve : Standardized version of the profile (makes it unique, up to permutations of the candidates).
+        """ProfileHistogram : Standardized version of the profile (makes it unique, up to permutations of the candidates).
 
          Examples
          --------
@@ -291,7 +328,8 @@ class ProfileHistogram(ProfileCardinal):
             d_ranking_share={ranking: best_d_ranking_share[xyz_ranking]
                              for ranking, xyz_ranking in zip(RANKINGS, XYZ_RANKINGS)},
             d_ranking_histogram={ranking: best_d_ranking_histogram[xyz_ranking]
-                                 for ranking, xyz_ranking in zip(RANKINGS, XYZ_RANKINGS)}
+                                 for ranking, xyz_ranking in zip(RANKINGS, XYZ_RANKINGS)},
+            ratio_sincere=self.ratio_sincere, ratio_fanatic=self.ratio_fanatic, voting_rule=self.voting_rule
         )
 
     def plot_cdf(self, ranking, x_label=None, y_label=None, **kwargs):
