@@ -2,14 +2,17 @@ import warnings
 import numpy as np
 from math import isclose
 from poisson_approval.constants.constants import *
+from poisson_approval.constants.EquilibriumStatus import EquilibriumStatus
+from poisson_approval.containers.AnalyzedStrategies import AnalyzedStrategies
+from poisson_approval.containers.Winners import Winners
 from poisson_approval.profiles.Profile import Profile
-from poisson_approval.utils.UtilMasks import masks_area, masks_distribution, winners_distribution
 from poisson_approval.strategies.StrategyOrdinal import StrategyOrdinal
 from poisson_approval.tau_vector.TauVector import TauVector
-from poisson_approval.containers.AnalyzedStrategies import AnalyzedStrategies
 from poisson_approval.utils.DictPrintingInOrderIgnoringZeros import DictPrintingInOrderIgnoringZeros
-from poisson_approval.constants.EquilibriumStatus import EquilibriumStatus
-from poisson_approval.utils.UtilCache import cached_property
+from poisson_approval.utils.Util import ballot_one, ballot_two, ballot_one_two, ballot_one_three, barycenter, \
+    product_dict, ballot_low_u, ballot_high_u
+from poisson_approval.utils.UtilCache import cached_property, property_deleting_cache
+from poisson_approval.utils.UtilMasks import masks_area, masks_distribution, winners_distribution
 
 
 # noinspection PyUnresolvedReferences
@@ -25,7 +28,12 @@ class ProfileOrdinal(Profile):
         Whether a warning should be issued if the input distribution is not normalized.
     well_informed_voters : bool.
         If True (default), it is the usual model. If False, voters "see" only the candidates' expected scores and
-        believe that the scores follow independent Poisson distributions.
+        believe that the scores follow independent Poisson distributions. This option has an effect only for Approval
+        (neither for Plurality nor Anti-plurality).
+    ratio_fanatic : Number
+        The ratio of fanatic voters, in the interval [0, 1]. This is used for :meth:`tau`.
+    voting_rule : str
+        The voting rule. Possible values are ``APPROVAL``, ``PLURALITY`` and ``ANTI_PLURALITY``.
 
     Notes
     -----
@@ -64,44 +72,66 @@ class ProfileOrdinal(Profile):
         False
     """
 
-    def __init__(self, d_ranking_share, normalization_warning=True, well_informed_voters=True):
-        """
-            >>> profile = ProfileOrdinal({'non_existing_ranking': 1})
-            Traceback (most recent call last):
-            ValueError: Unknown key: non_existing_ranking
-        """
-        super().__init__()
-        # Populate the dictionary and check for typos in the input
-        self._d_ranking_share = DictPrintingInOrderIgnoringZeros()
+    def __init__(self, d_ranking_share, normalization_warning=True, well_informed_voters=True, ratio_fanatic=0,
+                 voting_rule=APPROVAL):
+        super().__init__(voting_rule=voting_rule)
+        # Populate the dictionary
+        self._d_ranking_share = DictPrintingInOrderIgnoringZeros({ranking: 0 for ranking in RANKINGS})
         for ranking, share in d_ranking_share.items():
-            if ranking in RANKINGS:
-                self._d_ranking_share[ranking] = share
-            else:
-                raise ValueError('Unknown key: ' + ranking)
-        for ranking in RANKINGS:
-            if ranking not in self._d_ranking_share:
-                self._d_ranking_share[ranking] = 0
+            self._d_ranking_share[ranking] += share
         # Normalize if necessary
         total = sum(self._d_ranking_share.values())
         if not isclose(total, 1.):
             if normalization_warning:
                 warnings.warn("Warning: profile is not normalized, I will normalize it.")
             for ranking in self._d_ranking_share.keys():
-                self._d_ranking_share[ranking] = self._d_ranking_share[ranking] / total
-        # Well-informed voters?
+                self._d_ranking_share[ranking] /= total
+        # Other parameters
         self.well_informed_voters = well_informed_voters
+        self.ratio_fanatic = ratio_fanatic
+
+    well_informed_voters = property_deleting_cache('_well_informed_voters')
+    ratio_fanatic = property_deleting_cache('_ratio_fanatic')
 
     @cached_property
     def d_ranking_share(self):
         return self._d_ranking_share
 
     def __repr__(self):
-        return 'ProfileOrdinal(%r)' % self._d_ranking_share
+        """
+            >>> from fractions import Fraction
+            >>> profile = ProfileOrdinal({'abc': Fraction(1, 10), 'bac': Fraction(6, 10), 'cab': Fraction(3, 10)},
+            ...                          well_informed_voters=False, ratio_fanatic=Fraction(1, 10))
+            >>> profile
+            ProfileOrdinal({'abc': Fraction(1, 10), 'bac': Fraction(3, 5), 'cab': Fraction(3, 10)}, \
+well_informed_voters=False, ratio_fanatic=Fraction(1, 10))
+        """
+        arguments = repr(self._d_ranking_share)
+        if not self.well_informed_voters:
+            arguments += ', well_informed_voters=False'
+        if self.ratio_fanatic > 0:
+            arguments += ', ratio_fanatic=%r' % self.ratio_fanatic
+        if self.voting_rule != APPROVAL:
+            arguments += ', voting_rule=%r' % self.voting_rule
+        return 'ProfileOrdinal(%s)' % arguments
 
     def __str__(self):
+        """
+            >>> from fractions import Fraction
+            >>> profile = ProfileOrdinal({'abc': Fraction(1, 10), 'bac': Fraction(6, 10), 'cab': Fraction(3, 10)},
+            ...                          well_informed_voters=False, ratio_fanatic=Fraction(1, 10))
+            >>> print(profile)
+            <abc: 1/10, bac: 3/5, cab: 3/10> (Condorcet winner: b) (badly informed voters) (ratio_fanatic: 1/10)
+        """
         result = '<%s>' % str(self._d_ranking_share)[1:-1]
         if self.is_profile_condorcet:
             result += ' (Condorcet winner: %s)' % self.condorcet_winners
+        if not self.well_informed_voters:
+            result += ' (badly informed voters)'
+        if self.ratio_fanatic > 0:
+            result += ' (ratio_fanatic: %s)' % self.ratio_fanatic
+        if self.voting_rule != APPROVAL:
+            result += ' (%s)' % self.voting_rule
         return result
 
     def _repr_pretty_(self, p, cycle):  # pragma: no cover
@@ -127,7 +157,11 @@ class ProfileOrdinal(Profile):
             >>> profile == ProfileOrdinal({'abc': Fraction(1, 10), 'bac': Fraction(6, 10), 'cab': Fraction(3, 10)})
             True
         """
-        return isinstance(other, ProfileOrdinal) and self.d_ranking_share == other.d_ranking_share
+        return (isinstance(other, ProfileOrdinal)
+                and self.d_ranking_share == other.d_ranking_share
+                and self.well_informed_voters == other.well_informed_voters
+                and self.ratio_fanatic == other.ratio_fanatic
+                and self.voting_rule == other.voting_rule)
 
     @cached_property
     def standardized_version(self):
@@ -153,7 +187,9 @@ class ProfileOrdinal(Profile):
             if signature_test > best_signature:
                 best_signature = signature_test
                 best_d = d_test
-        return ProfileOrdinal({ranking: best_d[xyz_ranking] for ranking, xyz_ranking in zip(RANKINGS, XYZ_RANKINGS)})
+        return ProfileOrdinal({ranking: best_d[xyz_ranking] for ranking, xyz_ranking in zip(RANKINGS, XYZ_RANKINGS)},
+                              well_informed_voters=self.well_informed_voters, ratio_fanatic=self.ratio_fanatic,
+                              voting_rule=self.voting_rule)
 
     # Has full support
     @cached_property
@@ -185,6 +221,62 @@ class ProfileOrdinal(Profile):
     # Tau and strategy-related stuff
 
     def tau(self, strategy):
+        """Tau-vector associated to a strategy, with partial fanatic voting.
+
+        Parameters
+        ----------
+        strategy : an argument accepted by :meth:`tau_strategic`
+
+        Returns
+        -------
+        TauVector
+            A share :attr:`ratio_fanatic` of voters vote only for their top candidate, and the rest of the voters
+            vote strategically (in the sense of :meth:`tau_strategic`). In other words, this tau-vector
+            is the barycenter of ``tau_fanatic`` and ``tau_strategic(strategy)``, with respective
+            weights ``self.ratio_fanatic`` and ``1 - self.ratio_fanatic``.
+
+        Examples
+        --------
+            >>> from fractions import Fraction
+            >>> profile = ProfileOrdinal({'abc': Fraction(1, 10), 'bac': Fraction(6, 10), 'cab': Fraction(3, 10)})
+            >>> strategy = StrategyOrdinal({'abc': 'a', 'bac': 'ab', 'cab': 'c'})
+            >>> tau = profile.tau(strategy)
+            >>> print(tau)
+            <a: 1/10, ab: 3/5, c: 3/10> ==> a
+            >>> τ = profile.τ(strategy)  # Alternate syntax
+            >>> print(τ)
+            <a: 1/10, ab: 3/5, c: 3/10> ==> a
+        """
+        tau_fanatic = self.tau_fanatic
+        tau_strategic = self.tau_strategic(strategy)
+        t = {ballot: barycenter(a=tau_strategic.d_ballot_share[ballot],
+                                b=tau_fanatic.d_ballot_share[ballot],
+                                ratio_b=self.ratio_fanatic)
+             for ballot in BALLOTS_WITHOUT_INVERSIONS}
+        return TauVector(t, voting_rule=self.voting_rule)
+
+    @cached_property
+    def tau_fanatic(self):
+        """Tau-vector associated to fanatic voting.
+
+        Returns
+        -------
+        TauVector
+            In Approval or Plurality, all voters approve of their top candidate only. In Anti-Plurality, they all
+            disapprove of their bottom candidate, i.e. they approve their two first candidates.
+        """
+        t = {ballot: 0 for ballot in BALLOTS_WITHOUT_INVERSIONS}
+        if self.voting_rule in {APPROVAL, PLURALITY}:
+            for ranking, share in self.d_ranking_share.items():
+                t[ballot_one(ranking)] += share
+        elif self.voting_rule == ANTI_PLURALITY:
+            for ranking, share in self.d_ranking_share.items():
+                t[ballot_one_two(ranking)] += share
+        else:
+            raise NotImplementedError
+        return TauVector(t, voting_rule=self.voting_rule)
+
+    def tau_strategic(self, strategy):
         """Tau-vector associated to a strategy.
 
         Parameters
@@ -202,34 +294,32 @@ class ProfileOrdinal(Profile):
             >>> from fractions import Fraction
             >>> profile = ProfileOrdinal({'abc': Fraction(1, 10), 'bac': Fraction(6, 10), 'cab': Fraction(3, 10)})
             >>> strategy = StrategyOrdinal({'abc': 'a', 'bac': 'ab', 'cab': 'c'})
-            >>> tau = profile.tau(strategy)
-            >>> print(tau)
-            <a: 1/10, ab: 3/5, c: 3/10> ==> a
-            >>> tau = profile.τ(strategy)  # Alternate notation
-            >>> print(tau)
+            >>> tau_strategic = profile.tau_strategic(strategy)
+            >>> print(tau_strategic)
             <a: 1/10, ab: 3/5, c: 3/10> ==> a
 
-        In the case of badly informed voters, we do as if the tau-vector were the vector of scores (up to
-        a renormalization):
+        In the case of approval with badly informed voters, we do as if the tau-vector were the vector of scores (up
+        to a renormalization):
 
             >>> from fractions import Fraction
             >>> profile = ProfileOrdinal({'abc': Fraction(1, 10), 'bac': Fraction(6, 10), 'cab': Fraction(3, 10)},
             ...                          well_informed_voters=False)
             >>> strategy = StrategyOrdinal({'abc': 'a', 'bac': 'ab', 'cab': 'c'})
-            >>> tau = profile.tau(strategy)
-            >>> print(tau)
+            >>> tau_strategic = profile.tau_strategic(strategy)
+            >>> print(tau_strategic)
             <a: 7/16, b: 3/8, c: 3/16> ==> a
         """
+        assert self.voting_rule == strategy.voting_rule
         t = {ballot: 0 for ballot in BALLOTS_WITHOUT_INVERSIONS}
         for ranking, ballot in strategy.d_ranking_ballot.items():
             if self.d_ranking_share[ranking] > 0:
                 t[ballot] += self.d_ranking_share[ranking]
-        if self.well_informed_voters:
-            return TauVector(t)
-        else:
+        if self.voting_rule == APPROVAL and not self.well_informed_voters:
             return TauVector(
                 {'a': t['a'] + t['ab'] + t['ac'], 'b': t['b'] + t['ab'] + t['bc'], 'c': t['c'] + t['ac'] + t['bc']},
-                normalization_warning=False)
+                normalization_warning=False, voting_rule=self.voting_rule)
+        else:
+            return TauVector(t, voting_rule=self.voting_rule)
 
     def is_equilibrium(self, strategy):
         """Whether a strategy is an equilibrium.
@@ -290,36 +380,30 @@ class ProfileOrdinal(Profile):
             <abc: ab, bac: ab, cab: ac> ==> a (D)
             >>> print(profile.analyzed_strategies.equilibria[0])
             <abc: a, bac: b, cab: ac> ==> b
+            >>> print(profile.winners_at_equilibrium)
+            a, b
         """
         equilibria = []
         utility_dependent = []
         inconclusive = []
         non_equilibria = []
-        strat_abc = ['a', 'ab'] if self.abc > 0 else ['']
-        strat_acb = ['a', 'ac'] if self.acb > 0 else ['']
-        strat_bac = ['b', 'ab'] if self.bac > 0 else ['']
-        strat_bca = ['b', 'bc'] if self.bca > 0 else ['']
-        strat_cab = ['c', 'ac'] if self.cab > 0 else ['']
-        strat_cba = ['c', 'bc'] if self.cba > 0 else ['']
-        for s_abc in strat_abc:
-            for s_acb in strat_acb:
-                for s_bac in strat_bac:
-                    for s_bca in strat_bca:
-                        for s_cab in strat_cab:
-                            for s_cba in strat_cba:
-                                strategy = StrategyOrdinal({'abc': s_abc, 'acb': s_acb, 'bac': s_bac,
-                                                         'bca': s_bca, 'cab': s_cab, 'cba': s_cba}, profile=self)
-                                status = strategy.is_equilibrium
-                                if status == EquilibriumStatus.EQUILIBRIUM:
-                                    equilibria.append(strategy)
-                                elif status == EquilibriumStatus.UTILITY_DEPENDENT:
-                                    utility_dependent.append(strategy)
-                                elif status == EquilibriumStatus.INCONCLUSIVE:  # pragma: no cover
-                                    inconclusive.append(strategy)
-                                    warnings.warn('Met an inconclusive case: \nprofile = %r\nstrategy = %r'
-                                                  % (self, strategy))
-                                else:
-                                    non_equilibria.append(strategy)
+        d_ranking_possible_ballots = {ranking: [ballot_low_u(ranking, self.voting_rule),
+                                                ballot_high_u(ranking, self.voting_rule)]
+                                      if self.d_ranking_share[ranking] > 0 else ['']
+                                      for ranking in RANKINGS}
+        for d_ranking_ballot in product_dict(d_ranking_possible_ballots):
+            strategy = StrategyOrdinal(d_ranking_ballot, profile=self, voting_rule=self.voting_rule)
+            status = strategy.is_equilibrium
+            if status == EquilibriumStatus.EQUILIBRIUM:
+                equilibria.append(strategy)
+            elif status == EquilibriumStatus.UTILITY_DEPENDENT:
+                utility_dependent.append(strategy)
+            elif status == EquilibriumStatus.INCONCLUSIVE:  # pragma: no cover
+                inconclusive.append(strategy)
+                warnings.warn('Met an inconclusive case: \nprofile = %r\nstrategy = %r'
+                              % (self, strategy))
+            else:
+                non_equilibria.append(strategy)
         return AnalyzedStrategies(equilibria, utility_dependent, inconclusive, non_equilibria)
 
     def proba_equilibrium(self, test=None):
