@@ -5,8 +5,11 @@ from fractions import Fraction
 from poisson_approval.constants.constants import *
 from poisson_approval.utils.Util import sort_ballot
 from poisson_approval.containers.Scores import Scores
+from poisson_approval.utils.DictPrintingInOrder import DictPrintingInOrder
 from poisson_approval.utils.DictPrintingInOrderIgnoringZeros import DictPrintingInOrderIgnoringZeros
-from poisson_approval.best_response.BestResponse import BestResponse
+from poisson_approval.best_response.BestResponseApproval import BestResponseApproval
+from poisson_approval.best_response.BestResponsePlurality import BestResponsePlurality
+from poisson_approval.best_response.BestResponseAntiPlurality import BestResponseAntiPlurality
 from poisson_approval.utils.UtilCache import cached_property
 from poisson_approval.events.EventDuo import EventDuo
 from poisson_approval.events.EventPivotStrict import EventPivotStrict
@@ -159,7 +162,8 @@ phi_ab = 0.707107>
         <asymptotic = exp(- 0.151472 n + 0.5 log n - 3.1394 + o(1)), phi_a = 0, phi_c = 1.41421, phi_ab = 0.707107>
     """
 
-    def __init__(self, d_ballot_share: dict, normalization_warning: bool = True):
+    def __init__(self, d_ballot_share: dict, normalization_warning: bool = True,
+                 voting_rule=APPROVAL):
         """
             >>> tau = TauVector({'non_existing_ballot': 1})
             Traceback (most recent call last):
@@ -178,17 +182,29 @@ phi_ab = 0.707107>
                 self.d_ballot_share[ballot] = 0
         # Normalize if necessary
         total = sum(self.d_ballot_share.values())
-        if not isclose(total, 1.):
+        if not isclose(total, 1., rel_tol=10E-4):
             if normalization_warning:
                 warnings.warn("Warning: tau is not normalized, I will normalize it.")
             for ballot in self.d_ballot_share.keys():
                 self.d_ballot_share[ballot] = self.d_ballot_share[ballot] / total
+        # Voting rule
+        self.voting_rule = voting_rule
+        if self.voting_rule == PLURALITY:
+            assert self.ab == self.ac == self.bc == 0
+        elif self.voting_rule == ANTI_PLURALITY:
+            assert self.a == self.b == self.c == 0
 
     def __repr__(self):
-        return 'TauVector(%r)' % self.d_ballot_share
+        arguments = repr(self.d_ballot_share)
+        if self.voting_rule != APPROVAL:
+            arguments += ', voting_rule=%r' % self.voting_rule
+        return 'TauVector(%s)' % arguments
 
     def __str__(self):
-        return '<%s>' % str(self.d_ballot_share)[1:-1] + ' ==> ' + str(self.winners)
+        s = '<%s>' % str(self.d_ballot_share)[1:-1] + ' ==> ' + str(self.winners)
+        if self.voting_rule != APPROVAL:
+            s += ' (%s)' % self.voting_rule
+        return s
 
     def _repr_pretty_(self, p, cycle):  # pragma: no cover
         # https://stackoverflow.com/questions/41453624/tell-ipython-to-use-an-objects-str-instead-of-repr-for-output
@@ -241,7 +257,20 @@ phi_ab = 0.707107>
             >>> tau == TauVector({'a': Fraction(10, 100), 'ab': Fraction(60, 100), 'c': Fraction(30, 100)})
             True
         """
-        return isinstance(other, TauVector) and self.d_ballot_share == other.d_ballot_share
+        return (isinstance(other, TauVector)
+                and self.d_ballot_share == other.d_ballot_share
+                and self.voting_rule == other.voting_rule)
+
+    @cached_property
+    def has_two_consecutive_zeros(self):
+        """bool
+
+        Whether the tau-vector has two consecutive holes in the "compass" representation. True iff
+        ``self.a == 0 and self.ab == 0``, or ``self.ab == 0 and self.b == 0``, etc.
+        """
+        return ((self.a == 0 and (self.ab == 0 or self.ac == 0))
+                or (self.b == 0 and (self.ab == 0 or self.bc == 0))
+                or (self.c == 0 and (self.ac == 0 or self.bc == 0)))
 
     def isclose(self, other, *args, **kwargs):
         """Test near-equality.
@@ -299,7 +328,8 @@ phi_ab = 0.707107>
                 best_signature = signature_test
                 best_d = d_test
         return TauVector({ballot: best_d[xyz_ballot]
-                          for ballot, xyz_ballot in zip(BALLOTS_WITHOUT_INVERSIONS, XYZ_BALLOTS_WITHOUT_INVERSION)})
+                          for ballot, xyz_ballot in zip(BALLOTS_WITHOUT_INVERSIONS, XYZ_BALLOTS_WITHOUT_INVERSION)},
+                         voting_rule=self.voting_rule)
 
     @cached_property
     def is_standardized(self):
@@ -439,28 +469,133 @@ phi_ab = 0.707107>
         """dict : Best response profile.
 
         * Key: a ranking (e.g. ``'abc'``).
-        * Value: a :class:`BestResponse`.
+        * Value: a :class:`BestResponse` (whose subclass depends on :attr:`voting_rule`).
 
         Examples
         --------
             >>> from fractions import Fraction
             >>> tau = TauVector({'a': Fraction(1, 10), 'ab': Fraction(3, 5), 'c': Fraction(3, 10)})
             >>> tau.d_ranking_best_response['abc']
-            <ballot = a, threshold_utility = 1, justification = Asymptotic method, pivot_tij = exp(- 0.1 n + o(1)), \
-pivot_tjk = exp(- inf), trio_1t = exp(- inf), trio_2t = exp(- 0.151472 n - 0.5 log n - 1.18339 + o(1)), \
-trio = exp(- 0.151472 n - 0.5 log n - 0.836813 + o(1))>
+            <ballot = a, threshold_utility = 1, justification = Asymptotic method>
         """
-        _d_ranking_best_response = dict()
-        for ranking in RANKINGS:
-            _d_ranking_best_response[ranking] = BestResponse(
-                ranking,
-                pivot_tij=getattr(self, 'pivot_tij_' + ranking),
-                pivot_tjk=getattr(self, 'pivot_tjk_' + ranking),
-                trio_1t=getattr(self, 'trio_1t_' + ranking[0]),
-                trio_2t=getattr(self, 'trio_2t_' + ranking[0] + ranking[1]),
-                trio=self.trio,
-            )
-        return _d_ranking_best_response
+        if self.voting_rule == APPROVAL:
+            return DictPrintingInOrder({
+                ranking: BestResponseApproval(tau=self, ranking=ranking) for ranking in RANKINGS})
+        if self.voting_rule == PLURALITY:
+            return DictPrintingInOrder({
+                ranking: BestResponsePlurality(tau=self, ranking=ranking) for ranking in RANKINGS})
+        if self.voting_rule == ANTI_PLURALITY:
+            return DictPrintingInOrder({
+                ranking: BestResponseAntiPlurality(tau=self, ranking=ranking) for ranking in RANKINGS})
+        raise NotImplementedError
+
+    @cached_property
+    def score_ab_in_duo_ab(self):
+        """Number : Common score of `a` and `b` in duo `ab`."""
+        return (_multiply(self.a, self.duo_ab.phi_a)
+                + _multiply(self.ab, self.duo_ab.phi_ab)
+                + _multiply(self.ac, self.duo_ab.phi_ac))
+
+    @cached_property
+    def score_ac_in_duo_ac(self):
+        """Number : Common score of `a` and `c` in duo `ac`."""
+        return (_multiply(self.a, self.duo_ac.phi_a)
+                + _multiply(self.ab, self.duo_ac.phi_ab)
+                + _multiply(self.ac, self.duo_ac.phi_ac))
+
+    @cached_property
+    def score_bc_in_duo_bc(self):
+        """Number : Common score of `b` and `c` in duo `bc`."""
+        return (_multiply(self.b, self.duo_bc.phi_b)
+                + _multiply(self.ab, self.duo_bc.phi_ab)
+                + _multiply(self.bc, self.duo_bc.phi_bc))
+
+    @cached_property
+    def score_ba_in_duo_ba(self):
+        """Number : Alternate notation for :attr:`score_ab_in_duo_ab`."""
+        return self.score_ab_in_duo_ab
+
+    @cached_property
+    def score_ca_in_duo_ca(self):
+        """Number : Alternate notation for :attr:`score_ac_in_duo_ac`."""
+        return self.score_ac_in_duo_ac
+
+    @cached_property
+    def score_cb_in_duo_cb(self):
+        """Number : Alternate notation for :attr:`score_bc_in_duo_bc`."""
+        return self.score_bc_in_duo_bc
+
+    @cached_property
+    def score_c_in_duo_ab(self):
+        """Number : Score of `c` in duo `ab`."""
+        return (_multiply(self.c, self.duo_ab.phi_c)
+                + _multiply(self.ac, self.duo_ab.phi_ac)
+                + _multiply(self.bc, self.duo_ab.phi_bc))
+
+    @cached_property
+    def score_b_in_duo_ac(self):
+        """Number : Score of `b` in duo `ac`."""
+        return (_multiply(self.b, self.duo_ac.phi_b)
+                + _multiply(self.ab, self.duo_ac.phi_ab)
+                + _multiply(self.bc, self.duo_ac.phi_bc))
+
+    @cached_property
+    def score_a_in_duo_bc(self):
+        """Number : Score of `a` in duo `bc`."""
+        return (_multiply(self.a, self.duo_bc.phi_a)
+                + _multiply(self.ab, self.duo_bc.phi_ab)
+                + _multiply(self.ac, self.duo_bc.phi_ac))
+
+    @cached_property
+    def score_c_in_duo_ba(self):
+        """Number : Alternate notation for :attr:`score_c_in_duo_ab`."""
+        return self.score_c_in_duo_ab
+
+    @cached_property
+    def score_b_in_duo_ca(self):
+        """Number : Alternate notation for :attr:`score_b_in_duo_ac`."""
+        return self.score_b_in_duo_ac
+
+    @cached_property
+    def score_a_in_duo_cb(self):
+        """Number : Alternate notation for :attr:`score_a_in_duo_bc`."""
+        return self.score_a_in_duo_bc
+
+    @cached_property
+    def pivot_ab_easy_or_tight(self):
+        """bool : True if the pivot `ab` is easy or tight, False if it is difficult."""
+        pivot_easy = self.score_ab_in_duo_ab > self.score_c_in_duo_ab
+        pivot_tight = isclose(self.score_ab_in_duo_ab, self.score_c_in_duo_ab)
+        return pivot_easy or pivot_tight
+
+    @cached_property
+    def pivot_ac_easy_or_tight(self):
+        """bool : True if the pivot `ac` is easy or tight, False if it is difficult."""
+        pivot_easy = self.score_ac_in_duo_ac > self.score_b_in_duo_ac
+        pivot_tight = isclose(self.score_ac_in_duo_ac, self.score_b_in_duo_ac)
+        return pivot_easy or pivot_tight
+
+    @cached_property
+    def pivot_bc_easy_or_tight(self):
+        """bool : True if the pivot `bc` is easy or tight, False if it is difficult."""
+        pivot_easy = self.score_bc_in_duo_bc > self.score_a_in_duo_bc
+        pivot_tight = isclose(self.score_bc_in_duo_bc, self.score_a_in_duo_bc)
+        return pivot_easy or pivot_tight
+
+    @cached_property
+    def pivot_ba_easy_or_tight(self):
+        """bool : Alternate notation for :attr:`pivot_ab_easy_or_tight`"""
+        return self.pivot_ab_easy_or_tight
+
+    @cached_property
+    def pivot_ca_easy_or_tight(self):
+        """bool : Alternate notation for :attr:`pivot_ac_easy_or_tight`"""
+        return self.pivot_ac_easy_or_tight
+
+    @cached_property
+    def pivot_cb_easy_or_tight(self):
+        """bool : Alternate notation for :attr:`pivot_bc_easy_or_tight`"""
+        return self.pivot_bc_easy_or_tight
 
 
 def _f_ballot_share(self, ballot):
@@ -542,3 +677,10 @@ for event_class, event_stub, event_doc in [
         getattr(TauVector, name).__name__ = name
         setattr(TauVector, name, cached_property(getattr(TauVector, name)))
         getattr(TauVector, name).__doc__ = event_doc
+
+
+# Used to compute easy and difficult pivots
+
+def _multiply(tau_something, phi_something):
+    """Return 0 if tau_something is 0, even if phi_something is nan."""
+    return 0 if tau_something == 0 else tau_something * phi_something
